from typing import (
    Any,
    Callable,
    Dict,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
    overload,
)

from faker import Faker
from faker.generator import Generator
from faker.providers import BaseProvider
from faker.providers.python import Provider

from ...base import (
    DEFAULT_FORMAT_FUNC,
    BytesValue,
    DynamicTemplate,
    FileMixin,
    StringValue,
)
from ...constants import DEFAULT_TEXT_MAX_NB_CHARS
from ...helpers import load_class_from_path
from ...registry import FILE_REGISTRY
from ...storages.base import BaseStorage
from ...storages.filesystem import FileSystemStorage
from ..base.pdf_generator import BasePdfGenerator
from ..mixins.graphic_image_mixin import GraphicImageMixin

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "DEFAULT_PDF_GENERATOR",
    "GraphicPdfFileProvider",
    "PDFKIT_PDF_GENERATOR",
    "PIL_PDF_GENERATOR",
    "PdfFileProvider",
    "REPORTLAB_PDF_GENERATOR",
)

PDFKIT_PDF_GENERATOR = (
    "faker_file.providers.pdf_file.generators.pdfkit_generator"
    ".PdfkitPdfGenerator"
)
PIL_PDF_GENERATOR = (
    "faker_file.providers.pdf_file.generators.pil_generator.PilPdfGenerator"
)
REPORTLAB_PDF_GENERATOR = (
    "faker_file.providers.pdf_file.generators.reportlab_generator"
    ".ReportlabPdfGenerator"
)
DEFAULT_PDF_GENERATOR = PDFKIT_PDF_GENERATOR


class PdfFileProvider(BaseProvider, FileMixin):
    """PDF file provider.

    Usage example:

    .. code-block:: python

        from faker import Faker
        from faker_file.providers.pdf_file import PdfFileProvider

        FAKER = Faker()
        FAKER.add_provider(PdfFileProvider)

        file = FAKER.pdf_file()

    Usage example with options:

    .. code-block:: python

        file = FAKER.pdf_file(
            prefix="zzz",
            max_nb_chars=100_000,
            wrap_chars_after=80,
        )

    Usage example with `FileSystemStorage` storage (for `Django`):

    .. code-block:: python

        from django.conf import settings
        from faker_file.storages.filesystem import FileSystemStorage

        file = FAKER.pdf_file(
            storage=FileSystemStorage(
                root_path=settings.MEDIA_ROOT,
                rel_path="tmp",
            ),
            prefix="zzz",
            max_nb_chars=100_000,
            wrap_chars_after=80,
        )

    Default PDF generator class is `PdfkitPdfGenerator` which uses `pdfkit`
    Python package and `wkhtmltopdf` system package for generating PDFs from
    randomly generated text. The quality of the produced PDFs is very good,
    but it's less performant than `ReportlabPdfGenerator` (factor 40x), which
    does not require additional system dependencies to run. To use it, pass
    `ReportlabPdfGenerator` class in `pdf_generator_cls` argument.

    .. code-block:: python

        from faker_file.providers.pdf_file.generators import (
            reportlab_generator,
        )

        file = FAKER.pdf_file(
            max_nb_chars=1_000,
            wrap_chars_after=80,
            pdf_generator_cls=reportlab_generator.ReportlabPdfGenerator,
        )
    """

    extension: str = "pdf"

    @overload
    def pdf_file(
        self: "PdfFileProvider",
        storage: Optional[BaseStorage] = None,
        basename: Optional[str] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[Union[str, DynamicTemplate]] = None,
        pdf_generator_cls: Optional[Union[str, Type[BasePdfGenerator]]] = (
            DEFAULT_PDF_GENERATOR
        ),
        pdf_generator_kwargs: Optional[Dict[str, Any]] = None,
        format_func: Callable[
            [Union[Faker, Generator, Provider], str], str
        ] = DEFAULT_FORMAT_FUNC,
        raw: bool = True,
        **kwargs,
    ) -> BytesValue: ...

    @overload
    def pdf_file(
        self: "PdfFileProvider",
        storage: Optional[BaseStorage] = None,
        basename: Optional[str] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[Union[str, DynamicTemplate]] = None,
        pdf_generator_cls: Optional[Union[str, Type[BasePdfGenerator]]] = (
            DEFAULT_PDF_GENERATOR
        ),
        pdf_generator_kwargs: Optional[Dict[str, Any]] = None,
        format_func: Callable[
            [Union[Faker, Generator, Provider], str], str
        ] = DEFAULT_FORMAT_FUNC,
        **kwargs,
    ) -> StringValue: ...

    def pdf_file(
        self: "PdfFileProvider",
        storage: Optional[BaseStorage] = None,
        basename: Optional[str] = None,
        prefix: Optional[str] = None,
        max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
        wrap_chars_after: Optional[int] = None,
        content: Optional[Union[str, DynamicTemplate]] = None,
        pdf_generator_cls: Optional[Union[str, Type[BasePdfGenerator]]] = (
            DEFAULT_PDF_GENERATOR
        ),
        pdf_generator_kwargs: Optional[Dict[str, Any]] = None,
        format_func: Callable[
            [Union[Faker, Generator, Provider], str], str
        ] = DEFAULT_FORMAT_FUNC,
        raw: bool = False,
        **kwargs,
    ) -> Union[BytesValue, StringValue]:
        """Generate a PDF file with random text.

        :param storage: Storage. Defaults to `FileSystemStorage`.
        :param basename: File basename (without extension).
        :param prefix: File name prefix.
        :param max_nb_chars: Max number of chars for the content.
        :param wrap_chars_after: If given, the output string would be separated
             by line breaks after the given position.
        :param content: File content. Might contain dynamic elements, which
            are then replaced by correspondent fixtures.
        :param pdf_generator_cls: PDF generator class.
        :param pdf_generator_kwargs: PDF generator kwargs.
        :param format_func: Callable responsible for formatting template
            strings.
        :param raw: If set to True, return `BytesValue` (binary content of
            the file). Otherwise, return `StringValue` (path to the saved
            file).
        :return: Relative path (from root directory) of the generated file
            or raw content of the file.
        """
        # Generic
        if storage is None:
            storage = FileSystemStorage()

        filename = storage.generate_filename(
            extension=self.extension,
            prefix=prefix,
            basename=basename,
        )

        if pdf_generator_cls is None:
            pdf_generator_cls = DEFAULT_PDF_GENERATOR

        if isinstance(pdf_generator_cls, str):
            pdf_generator_cls = load_class_from_path(pdf_generator_cls)

        if not pdf_generator_kwargs:
            pdf_generator_kwargs = {}

        pdf_generator_kwargs["content_specs"] = {
            "max_nb_chars": max_nb_chars,
            "wrap_chars_after": wrap_chars_after,
        }

        pdf_generator = pdf_generator_cls(
            generator=self.generator,
            **pdf_generator_kwargs,
        )
        data = {"content": "", "filename": filename, "storage": storage}
        if isinstance(content, DynamicTemplate):
            _content = content
        else:
            _content = self._generate_text_content(
                max_nb_chars=max_nb_chars,
                wrap_chars_after=wrap_chars_after,
                content=content,
                format_func=format_func,
            )
            data["content"] = _content

        _raw_content = pdf_generator.generate(
            content=_content,
            data=data,
            provider=self,
        )

        if raw:
            raw_content = BytesValue(_raw_content)
            raw_content.data = data
            return raw_content

        storage.write_bytes(filename, _raw_content)

        # Generic
        file_name = StringValue(storage.relpath(filename))
        file_name.data = data
        FILE_REGISTRY.add(file_name)
        return file_name


class GraphicPdfFileProvider(BaseProvider, GraphicImageMixin):
    """Graphic PDF file provider.

    Usage example:

    .. code-block:: python

        from faker import Faker
        from faker_file.providers.pdf_file import GraphicPdfFileProvider

        FAKER = Faker()
        FAKER.add_provider(GraphicPdfFileProvider)

        file = FAKER.graphic_pdf_file()

    Usage example with options:

    .. code-block:: python

        file = FAKER.graphic_pdf_file(
            prefix="zzz",
            size=(800, 800),
        )

    Usage example with `FileSystemStorage` storage (for `Django`):

    .. code-block:: python

        from django.conf import settings
        from faker_file.storages.filesystem import FileSystemStorage

        file = FAKER.graphic_pdf_file(
            storage=FileSystemStorage(
                root_path=settings.MEDIA_ROOT,
                rel_path="tmp",
            ),
            basename="yyy",
            size=(1024, 1024),
        )
    """

    extension: str = "pdf"
    image_format: str = "pdf"

    @overload
    def graphic_pdf_file(
        self: "GraphicPdfFileProvider",
        storage: Optional[BaseStorage] = None,
        basename: Optional[str] = None,
        prefix: Optional[str] = None,
        size: Tuple[int, int] = (256, 256),
        hue: Union[int, Sequence[int], str, None] = None,
        luminosity: Optional[str] = None,
        raw: bool = True,
        **kwargs,
    ) -> BytesValue: ...

    @overload
    def graphic_pdf_file(
        self: "GraphicPdfFileProvider",
        storage: Optional[BaseStorage] = None,
        basename: Optional[str] = None,
        prefix: Optional[str] = None,
        size: Tuple[int, int] = (256, 256),
        hue: Union[int, Sequence[int], str, None] = None,
        luminosity: Optional[str] = None,
        **kwargs,
    ) -> StringValue: ...

    def graphic_pdf_file(
        self: "GraphicPdfFileProvider",
        storage: Optional[BaseStorage] = None,
        basename: Optional[str] = None,
        prefix: Optional[str] = None,
        size: Tuple[int, int] = (256, 256),
        hue: Union[int, Sequence[int], str, None] = None,
        luminosity: Optional[str] = None,
        raw: bool = False,
        **kwargs,
    ) -> Union[BytesValue, StringValue]:
        """Generate a graphic PDF file with random lines.

        :param storage: Storage. Defaults to `FileSystemStorage`.
        :param basename: File basename (without extension).
        :param prefix: File name prefix.
        :param size: Image size in pixels.
        :param hue: Read more about
            ://faker.readthedocs.io/en/dev/providers/faker.providers.color.html
        :param luminosity: If given, the output string would be separated
             by line breaks after the given position.
        :param raw: If set to True, return `BytesValue` (binary content of
            the file). Otherwise, return `StringValue` (path to the saved
            file).
        :return: Relative path (from root directory) of the generated file
            or raw content of the file.
        """
        return self._image_file(
            image_format=self.image_format,
            storage=storage,
            basename=basename,
            prefix=prefix,
            size=size,
            hue=hue,
            luminosity=luminosity,
            raw=raw,
            **kwargs,
        )



from faker import Faker
from faker_file.providers.txt_file import TxtFileProvider

FAKER = Faker()
FAKER.add_provider(TxtFileProvider)

txt_file = FAKER.txt_file(max_nb_chars=1024**2)  # 1 Mb
txt_file = FAKER.txt_file(max_nb_chars=3 * 1024**2)  # 3 Mb
txt_file = FAKER.txt_file(max_nb_chars=10 * 1024**2)  # 10 Mb

txt_file = FAKER.txt_file(max_nb_chars=1024)  # 1 Kb
txt_file = FAKER.txt_file(max_nb_chars=3 * 1024)  # 3 Kb
txt_file = FAKER.txt_file(max_nb_chars=10 * 1024)  # 10 Kb



from random import choice

from factory import Faker, LazyAttribute, Sequence, Trait
from factory.alchemy import SQLAlchemyModelFactory
from faker_file.providers.bin_file import BinFileProvider
from faker_file.providers.csv_file import CsvFileProvider
from faker_file.providers.docx_file import DocxFileProvider
from faker_file.providers.eml_file import EmlFileProvider
from faker_file.providers.epub_file import EpubFileProvider
from faker_file.providers.ico_file import IcoFileProvider
from faker_file.providers.jpeg_file import JpegFileProvider
from faker_file.providers.mp3_file import Mp3FileProvider
from faker_file.providers.odp_file import OdpFileProvider
from faker_file.providers.ods_file import OdsFileProvider
from faker_file.providers.odt_file import OdtFileProvider
from faker_file.providers.pdf_file import PdfFileProvider
from faker_file.providers.png_file import PngFileProvider
from faker_file.providers.pptx_file import PptxFileProvider
from faker_file.providers.rtf_file import RtfFileProvider
from faker_file.providers.svg_file import SvgFileProvider
from faker_file.providers.tar_file import TarFileProvider
from faker_file.providers.txt_file import TxtFileProvider
from faker_file.providers.webp_file import WebpFileProvider
from faker_file.providers.xlsx_file import XlsxFileProvider
from faker_file.providers.zip_file import ZipFileProvider
from faker_file.storages.filesystem import FileSystemStorage

from faker_file_admin import db
from faker_file_admin.models import Upload

__all__ = ("UploadFactory",)

# Faker._DEFAULT_LOCALE = "hy_AM"
Faker.add_provider(BinFileProvider)
Faker.add_provider(CsvFileProvider)
Faker.add_provider(DocxFileProvider)
Faker.add_provider(EmlFileProvider)
Faker.add_provider(EpubFileProvider)
Faker.add_provider(IcoFileProvider)
Faker.add_provider(JpegFileProvider)
Faker.add_provider(Mp3FileProvider)
Faker.add_provider(OdpFileProvider)
Faker.add_provider(OdsFileProvider)
Faker.add_provider(OdtFileProvider)
Faker.add_provider(PdfFileProvider)
Faker.add_provider(PngFileProvider)
Faker.add_provider(PptxFileProvider)
Faker.add_provider(RtfFileProvider)
Faker.add_provider(SvgFileProvider)
Faker.add_provider(TarFileProvider)
Faker.add_provider(TxtFileProvider)
Faker.add_provider(WebpFileProvider)
Faker.add_provider(XlsxFileProvider)
Faker.add_provider(ZipFileProvider)


STORAGE = FileSystemStorage(root_path="", rel_path="tmp")


PROVIDER_CHOICES = [
    lambda: BinFileProvider(None).bin_file(storage=STORAGE),
    lambda: CsvFileProvider(None).csv_file(storage=STORAGE),
    lambda: DocxFileProvider(None).docx_file(storage=STORAGE),
    lambda: EmlFileProvider(None).eml_file(storage=STORAGE),
    lambda: EpubFileProvider(None).epub_file(storage=STORAGE),
    lambda: IcoFileProvider(None).ico_file(storage=STORAGE),
    lambda: JpegFileProvider(None).jpeg_file(storage=STORAGE),
    lambda: Mp3FileProvider(None).mp3_file(storage=STORAGE),
    lambda: OdpFileProvider(None).odp_file(storage=STORAGE),
    lambda: OdsFileProvider(None).ods_file(storage=STORAGE),
    lambda: OdtFileProvider(None).odt_file(storage=STORAGE),
    lambda: PdfFileProvider(None).pdf_file(storage=STORAGE),
    lambda: PngFileProvider(None).png_file(storage=STORAGE),
    lambda: PptxFileProvider(None).pptx_file(storage=STORAGE),
    lambda: RtfFileProvider(None).rtf_file(storage=STORAGE),
    lambda: SvgFileProvider(None).svg_file(storage=STORAGE),
    lambda: TarFileProvider(None).tar_file(storage=STORAGE),
    lambda: TxtFileProvider(None).txt_file(storage=STORAGE),
    lambda: XlsxFileProvider(None).xlsx_file(storage=STORAGE),
    lambda: ZipFileProvider(None).zip_file(storage=STORAGE),
]


def pick_random_provider(*args, **kwargs):
    return choice(PROVIDER_CHOICES)()


class AbstractUploadFactory(SQLAlchemyModelFactory):
    """Abstract Upload factory.

    Usage example:

        import sqlalchemy_factories as factories
        from faker_file_admin import app, db

        with app.app_context():
            document = factories.DocxUploadFactory()
            db.session.commit()
    """

    id = Sequence(lambda n: n)
    name = Faker("text", max_nb_chars=100)
    description = Faker("text", max_nb_chars=1000)

    class Meta:
        """Meta class."""

        model = Upload
        sqlalchemy_session = db.session  # the SQLAlchemy session object
        abstract = True


class UploadFactory(AbstractUploadFactory):
    """Upload factory."""

    class Params:
        bin_file = Trait(file=Faker("bin_file", storage=STORAGE))
        csv_file = Trait(file=Faker("csv_file", storage=STORAGE))
        docx_file = Trait(file=Faker("docx_file", storage=STORAGE))
        eml_file = Trait(file=Faker("eml_file", storage=STORAGE))
        epub_file = Trait(file=Faker("epub_file", storage=STORAGE))
        ico_file = Trait(file=Faker("ico_file", storage=STORAGE))
        jpeg_file = Trait(file=Faker("jpeg_file", storage=STORAGE))
        mp3_file = Trait(file=Faker("mp3_file", storage=STORAGE))
        odp_file = Trait(file=Faker("odp_file", storage=STORAGE))
        ods_file = Trait(file=Faker("ods_file", storage=STORAGE))
        odt_file = Trait(file=Faker("odt_file", storage=STORAGE))
        pdf_file = Trait(file=Faker("pdf_file", storage=STORAGE))
        png_file = Trait(file=Faker("png_file", storage=STORAGE))
        pptx_file = Trait(file=Faker("pptx_file", storage=STORAGE))
        rtf_file = Trait(file=Faker("rtf_file", storage=STORAGE))
        svg_file = Trait(file=Faker("svg_file", storage=STORAGE))
        tar_file = Trait(file=Faker("tar_file", storage=STORAGE))
        txt_file = Trait(file=Faker("txt_file", storage=STORAGE))
        webp_file = Trait(file=Faker("webp_file", storage=STORAGE))
        xlsx_file = Trait(file=Faker("xlsx_file", storage=STORAGE))
        zip_file = Trait(file=Faker("zip_file", storage=STORAGE))
        random_file = Trait(file=LazyAttribute(pick_random_provider))



from random import choice
from typing import (
    Any,
    Callable,
    Dict,
    Iterable,
    List,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
    overload,
)

from faker import Faker
from faker.generator import Generator
from faker.providers.python import Provider

from ...base import DEFAULT_FORMAT_FUNC, BytesValue, StringValue
from ...constants import (
    DEFAULT_AUDIO_MAX_NB_CHARS,
    DEFAULT_IMAGE_MAX_NB_CHARS,
    DEFAULT_TEXT_MAX_NB_CHARS,
)
from ...helpers import random_pop
from ...storages.base import BaseStorage
from ..base.mp3_generator import BaseMp3Generator
from ..base.pdf_generator import BasePdfGenerator

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "create_inner_augment_image_from_path",
    "create_inner_augment_random_image_from_dir",
    "create_inner_bin_file",
    "create_inner_csv_file",
    "create_inner_docx_file",
    "create_inner_eml_file",
    "create_inner_epub_file",
    "create_inner_file_from_path",
    "create_inner_generic_file",
    "create_inner_graphic_ico_file",
    "create_inner_graphic_jpeg_file",
    "create_inner_graphic_pdf_file",
    "create_inner_graphic_png_file",
    "create_inner_graphic_webp_file",
    "create_inner_ico_file",
    "create_inner_jpeg_file",
    "create_inner_json_file",
    "create_inner_mp3_file",
    "create_inner_odp_file",
    "create_inner_ods_file",
    "create_inner_odt_file",
    "create_inner_pdf_file",
    "create_inner_png_file",
    "create_inner_pptx_file",
    "create_inner_random_file_from_dir",
    "create_inner_rtf_file",
    "create_inner_svg_file",
    "create_inner_tar_file",
    "create_inner_txt_file",
    "create_inner_webp_file",
    "create_inner_xlsx_file",
    "create_inner_xml_file",
    "create_inner_zip_file",
    "fuzzy_choice_create_inner_file",
    "list_create_inner_file",
)


# ************************************************
# ************ augment_image_from_path ***********
# ************************************************


@overload
def create_inner_augment_image_from_path(
    path: str,
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    augmentations: Optional[List[Tuple[Callable, Dict[str, Any]]]] = None,
    num_steps: Optional[int] = None,
    pop_func: Callable = random_pop,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_augment_image_from_path(
    path: str,
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    augmentations: Optional[List[Tuple[Callable, Dict[str, Any]]]] = None,
    num_steps: Optional[int] = None,
    pop_func: Callable = random_pop,
    **kwargs,
) -> StringValue: ...


def create_inner_augment_image_from_path(
    path: str,
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    augmentations: Optional[List[Tuple[Callable, Dict[str, Any]]]] = None,
    num_steps: Optional[int] = None,
    pop_func: Callable = random_pop,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner augment_image_from_path file."""
    try:
        from ..augment_image_from_path import AugmentImageFromPathProvider
    except ImportError as err:
        raise err

    return AugmentImageFromPathProvider(generator).augment_image_from_path(
        path=path,
        storage=storage,
        basename=basename,
        prefix=prefix,
        augmentations=augmentations,
        num_steps=num_steps,
        pop_func=pop_func,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ******** augment_random_image_from_dir *********
# ************************************************


@overload
def create_inner_augment_random_image_from_dir(
    source_dir_path: str,
    extensions: Optional[Iterable[str]] = None,
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    augmentations: Optional[List[Tuple[Callable, Dict[str, Any]]]] = None,
    num_steps: Optional[int] = None,
    pop_func: Callable = random_pop,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_augment_random_image_from_dir(
    source_dir_path: str,
    extensions: Optional[Iterable[str]] = None,
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    augmentations: Optional[List[Tuple[Callable, Dict[str, Any]]]] = None,
    num_steps: Optional[int] = None,
    pop_func: Callable = random_pop,
    **kwargs,
) -> StringValue: ...


def create_inner_augment_random_image_from_dir(
    source_dir_path: str,
    extensions: Optional[Iterable[str]] = None,
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    augmentations: Optional[List[Tuple[Callable, Dict[str, Any]]]] = None,
    num_steps: Optional[int] = None,
    pop_func: Callable = random_pop,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner augment_random_image_from_dir file."""
    try:
        from ..augment_random_image_from_dir import (
            AugmentRandomImageFromDirProvider,
        )
    except ImportError as err:
        raise err

    return AugmentRandomImageFromDirProvider(
        generator
    ).augment_random_image_from_dir(
        source_dir_path=source_dir_path,
        extensions=extensions,
        storage=storage,
        basename=basename,
        prefix=prefix,
        augmentations=augmentations,
        num_steps=num_steps,
        pop_func=pop_func,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* BIN **********************
# ************************************************


@overload
def create_inner_bin_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    length: int = (1 * 1024 * 1024),
    content: Optional[bytes] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_bin_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    length: int = (1 * 1024 * 1024),
    content: Optional[bytes] = None,
    **kwargs,
) -> StringValue: ...


def create_inner_bin_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    length: int = (1 * 1024 * 1024),
    content: Optional[bytes] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner BIN file."""
    try:
        from ..bin_file import BinFileProvider
    except ImportError as err:
        raise err

    return BinFileProvider(generator).bin_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        length=length,
        content=content,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* CSV **********************
# ************************************************


@overload
def create_inner_csv_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    header: Optional[Sequence[str]] = None,
    data_columns: Tuple[str, str] = ("{{name}}", "{{address}}"),
    num_rows: int = 10,
    include_row_ids: bool = False,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_csv_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    header: Optional[Sequence[str]] = None,
    data_columns: Tuple[str, str] = ("{{name}}", "{{address}}"),
    num_rows: int = 10,
    include_row_ids: bool = False,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_csv_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    header: Optional[Sequence[str]] = None,
    data_columns: Tuple[str, str] = ("{{name}}", "{{address}}"),
    num_rows: int = 10,
    include_row_ids: bool = False,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner CSV file."""
    try:
        from ..csv_file import CsvFileProvider
    except ImportError as err:
        raise err

    return CsvFileProvider(generator).csv_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        header=header,
        data_columns=data_columns,
        num_rows=num_rows,
        include_row_ids=include_row_ids,
        content=content,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ******************** DOCX **********************
# ************************************************


@overload
def create_inner_docx_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_docx_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_docx_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner DOCX file."""
    try:
        from ..docx_file import DocxFileProvider
    except ImportError as err:
        raise err

    return DocxFileProvider(generator).docx_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        raw=raw,
        format_func=format_func,
        **kwargs,
    )


# ************************************************
# ********************* EML **********************
# ************************************************


@overload
def create_inner_eml_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_eml_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_eml_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner EML file."""
    try:
        from ..eml_file import EmlFileProvider
    except ImportError as err:
        raise err
    return EmlFileProvider(generator).eml_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        options=options,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ******************** EPUB **********************
# ************************************************


@overload
def create_inner_epub_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    title: Optional[str] = None,
    chapter_title: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_epub_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    title: Optional[str] = None,
    chapter_title: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_epub_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    title: Optional[str] = None,
    chapter_title: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner EPUB file."""
    try:
        from ..epub_file import EpubFileProvider
    except ImportError as err:
        raise err

    return EpubFileProvider(generator).epub_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        title=title,
        chapter_title=chapter_title,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ***************** FileFromPath *****************
# ************************************************


@overload
def create_inner_file_from_path(
    path: str,
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_file_from_path(
    path: str,
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    **kwargs,
) -> StringValue: ...


def create_inner_file_from_path(
    path: str,
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner file from path."""
    try:
        from ..file_from_path import FileFromPathProvider
    except ImportError as err:
        raise err

    return FileFromPathProvider(generator).file_from_path(
        path,
        storage=storage,
        basename=basename,
        prefix=prefix,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ******************** Generic *******************
# ************************************************


@overload
def create_inner_generic_file(
    content: Union[bytes, str],
    extension: str,
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_generic_file(
    content: Union[bytes, str],
    extension: str,
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_generic_file(
    content: Union[bytes, str],
    extension: str,
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner generic file."""
    try:
        from ..generic_file import GenericFileProvider
    except ImportError as err:
        raise err

    return GenericFileProvider(generator).generic_file(
        content=content,
        extension=extension,
        storage=storage,
        basename=basename,
        prefix=prefix,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* ICO **********************
# ************************************************

# Text-to-image ICO file


@overload
def create_inner_ico_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_ico_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_ico_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner ICO file."""
    try:
        from ..ico_file import IcoFileProvider
    except ImportError as err:
        raise err

    return IcoFileProvider(generator).ico_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# Graphic ICO file


@overload
def create_inner_graphic_ico_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    size: Tuple[int, int] = (256, 256),
    hue: Union[int, Sequence[int], str, None] = None,
    luminosity: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_graphic_ico_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    size: Tuple[int, int] = (256, 256),
    hue: Union[int, Sequence[int], str, None] = None,
    luminosity: Optional[str] = None,
    **kwargs,
) -> StringValue: ...


def create_inner_graphic_ico_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    size: Tuple[int, int] = (256, 256),
    hue: Union[int, Sequence[int], str, None] = None,
    luminosity: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner graphic ICO file."""
    try:
        from ..ico_file import GraphicIcoFileProvider
    except ImportError as err:
        raise err

    return GraphicIcoFileProvider(generator).graphic_ico_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        size=size,
        hue=hue,
        luminosity=luminosity,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ******************** JPEG **********************
# ************************************************


@overload
def create_inner_jpeg_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_jpeg_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_jpeg_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner JPEG file."""
    try:
        from ..jpeg_file import JpegFileProvider
    except ImportError as err:
        raise err

    return JpegFileProvider(generator).jpeg_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# Graphic JPEG file


@overload
def create_inner_graphic_jpeg_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    size: Tuple[int, int] = (256, 256),
    hue: Union[int, Sequence[int], str, None] = None,
    luminosity: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_graphic_jpeg_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    size: Tuple[int, int] = (256, 256),
    hue: Union[int, Sequence[int], str, None] = None,
    luminosity: Optional[str] = None,
    **kwargs,
) -> StringValue: ...


def create_inner_graphic_jpeg_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    size: Tuple[int, int] = (256, 256),
    hue: Union[int, Sequence[int], str, None] = None,
    luminosity: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner graphic JPEG file."""
    try:
        from ..jpeg_file import GraphicJpegFileProvider
    except ImportError as err:
        raise err

    return GraphicJpegFileProvider(generator).graphic_jpeg_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        size=size,
        hue=hue,
        luminosity=luminosity,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ******************** JSON **********************
# ************************************************


@overload
def create_inner_json_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    data_columns: Optional[List] = None,
    num_rows: int = 10,
    indent: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_json_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    data_columns: Optional[List] = None,
    num_rows: int = 10,
    indent: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_json_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    data_columns: Optional[List] = None,
    num_rows: int = 10,
    indent: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner JSON file."""
    try:
        from ..json_file import JsonFileProvider
    except ImportError as err:
        raise err

    return JsonFileProvider(generator).json_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        data_columns=data_columns,
        num_rows=num_rows,
        indent=indent,
        content=content,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* MP3 **********************
# ************************************************


@overload
def create_inner_mp3_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_AUDIO_MAX_NB_CHARS,
    content: Optional[str] = None,
    mp3_generator_cls: Optional[Union[str, Type[BaseMp3Generator]]] = None,
    mp3_generator_kwargs: Optional[Dict[str, Any]] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_mp3_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_AUDIO_MAX_NB_CHARS,
    content: Optional[str] = None,
    mp3_generator_cls: Optional[Union[str, Type[BaseMp3Generator]]] = None,
    mp3_generator_kwargs: Optional[Dict[str, Any]] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_mp3_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_AUDIO_MAX_NB_CHARS,
    content: Optional[str] = None,
    mp3_generator_cls: Optional[Union[str, Type[BaseMp3Generator]]] = None,
    mp3_generator_kwargs: Optional[Dict[str, Any]] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner ODS file."""
    try:
        from ..mp3_file import Mp3FileProvider
    except ImportError as err:
        raise err

    return Mp3FileProvider(generator).mp3_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        content=content,
        mp3_generator_cls=mp3_generator_cls,
        mp3_generator_kwargs=mp3_generator_kwargs,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* ODP **********************
# ************************************************


@overload
def create_inner_odp_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_odp_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> Union[BytesValue, StringValue]: ...


def create_inner_odp_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner ODP file."""
    try:
        from ..odp_file import OdpFileProvider
    except ImportError as err:
        raise err

    return OdpFileProvider(generator).odp_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* ODS **********************
# ************************************************


@overload
def create_inner_ods_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    data_columns: Optional[Dict[str, str]] = None,
    num_rows: int = 10,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_ods_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    data_columns: Optional[Dict[str, str]] = None,
    num_rows: int = 10,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_ods_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    data_columns: Optional[Dict[str, str]] = None,
    num_rows: int = 10,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner ODS file."""
    try:
        from ..ods_file import OdsFileProvider
    except ImportError as err:
        raise err

    return OdsFileProvider(generator).ods_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        data_columns=data_columns,
        num_rows=num_rows,
        content=content,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* ODT **********************
# ************************************************


@overload
def create_inner_odt_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_odt_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_odt_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner ODT file."""
    try:
        from ..odt_file import OdtFileProvider
    except ImportError as err:
        raise err

    return OdtFileProvider(generator).odt_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* PDF **********************
# ************************************************


@overload
def create_inner_pdf_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    pdf_generator_cls: Optional[Union[str, Type[BasePdfGenerator]]] = None,
    pdf_generator_kwargs: Optional[Dict[str, Any]] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_pdf_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    pdf_generator_cls: Optional[Union[str, Type[BasePdfGenerator]]] = None,
    pdf_generator_kwargs: Optional[Dict[str, Any]] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_pdf_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    pdf_generator_cls: Optional[Union[str, Type[BasePdfGenerator]]] = None,
    pdf_generator_kwargs: Optional[Dict[str, Any]] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner PDF file."""
    try:
        from ..pdf_file import PdfFileProvider
    except ImportError as err:
        raise err

    return PdfFileProvider(generator).pdf_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        pdf_generator_cls=pdf_generator_cls,
        pdf_generator_kwargs=pdf_generator_kwargs,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# Graphic PDF file


@overload
def create_inner_graphic_pdf_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    size: Tuple[int, int] = (256, 256),
    hue: Union[int, Sequence[int], str, None] = None,
    luminosity: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_graphic_pdf_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    size: Tuple[int, int] = (256, 256),
    hue: Union[int, Sequence[int], str, None] = None,
    luminosity: Optional[str] = None,
    **kwargs,
) -> StringValue: ...


def create_inner_graphic_pdf_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    size: Tuple[int, int] = (256, 256),
    hue: Union[int, Sequence[int], str, None] = None,
    luminosity: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner graphic PDF file."""
    try:
        from ..pdf_file import GraphicPdfFileProvider
    except ImportError as err:
        raise err

    return GraphicPdfFileProvider(generator).graphic_pdf_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        size=size,
        hue=hue,
        luminosity=luminosity,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* PNG **********************
# ************************************************


@overload
def create_inner_png_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_png_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_png_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner PNG file."""
    try:
        from ..png_file import PngFileProvider
    except ImportError as err:
        raise err

    return PngFileProvider(generator).png_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# Graphic PNG file


@overload
def create_inner_graphic_png_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    size: Tuple[int, int] = (256, 256),
    hue: Union[int, Sequence[int], str, None] = None,
    luminosity: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_graphic_png_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    size: Tuple[int, int] = (256, 256),
    hue: Union[int, Sequence[int], str, None] = None,
    luminosity: Optional[str] = None,
    **kwargs,
) -> StringValue: ...


def create_inner_graphic_png_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    size: Tuple[int, int] = (256, 256),
    hue: Union[int, Sequence[int], str, None] = None,
    luminosity: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner graphic PNG file."""
    try:
        from ..png_file import GraphicPngFileProvider
    except ImportError as err:
        raise err

    return GraphicPngFileProvider(generator).graphic_png_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        size=size,
        hue=hue,
        luminosity=luminosity,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ******************** PPTX **********************
# ************************************************


@overload
def create_inner_pptx_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_pptx_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_pptx_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner PPTX file."""
    try:
        from ..pptx_file import PptxFileProvider
    except ImportError as err:
        raise err

    return PptxFileProvider(generator).pptx_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# ************************************************
# *********** RandomFileFromDirProvider **********
# ************************************************


@overload
def create_inner_random_file_from_dir(
    source_dir_path: str,
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_random_file_from_dir(
    source_dir_path: str,
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    **kwargs,
) -> StringValue: ...


def create_inner_random_file_from_dir(
    source_dir_path: str,
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner random_file_from_dir file."""
    try:
        from faker_file.providers.random_file_from_dir import (
            RandomFileFromDirProvider,
        )
    except ImportError as err:
        raise err

    return RandomFileFromDirProvider(generator).random_file_from_dir(
        source_dir_path=source_dir_path,
        storage=storage,
        basename=basename,
        prefix=prefix,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* RTF **********************
# ************************************************


@overload
def create_inner_rtf_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_rtf_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_rtf_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner RTF file."""
    try:
        from ..rtf_file import RtfFileProvider
    except ImportError as err:
        raise err

    return RtfFileProvider(generator).rtf_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* SVG **********************
# ************************************************


@overload
def create_inner_svg_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_svg_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_svg_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner SVG file."""
    try:
        from ..svg_file import SvgFileProvider
    except ImportError as err:
        raise err

    return SvgFileProvider(generator).svg_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* TAR **********************
# ************************************************


@overload
def create_inner_tar_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    compression: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_tar_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    compression: Optional[str] = None,
    **kwargs,
) -> StringValue: ...


def create_inner_tar_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    compression: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner TAR file."""
    try:
        from ..tar_file import TarFileProvider
    except ImportError as err:
        raise err
    return TarFileProvider(generator).tar_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        options=options,
        compression=compression,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* TXT **********************
# ************************************************


@overload
def create_inner_txt_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_txt_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_txt_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_TEXT_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner TXT file."""
    try:
        from ..txt_file import TxtFileProvider
    except ImportError as err:
        raise err

    return TxtFileProvider(generator).txt_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ******************** WEBP **********************
# ************************************************


@overload
def create_inner_webp_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_webp_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_webp_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    max_nb_chars: int = DEFAULT_IMAGE_MAX_NB_CHARS,
    wrap_chars_after: Optional[int] = None,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner WEBP file."""
    try:
        from ..webp_file import WebpFileProvider
    except ImportError as err:
        raise err

    return WebpFileProvider(generator).webp_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        max_nb_chars=max_nb_chars,
        wrap_chars_after=wrap_chars_after,
        content=content,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# Graphic WEBP file


@overload
def create_inner_graphic_webp_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    size: Tuple[int, int] = (256, 256),
    hue: Union[int, Sequence[int], str, None] = None,
    luminosity: Optional[str] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_graphic_webp_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    size: Tuple[int, int] = (256, 256),
    hue: Union[int, Sequence[int], str, None] = None,
    luminosity: Optional[str] = None,
    **kwargs,
) -> StringValue: ...


def create_inner_graphic_webp_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    size: Tuple[int, int] = (256, 256),
    hue: Union[int, Sequence[int], str, None] = None,
    luminosity: Optional[str] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner graphic WEBP file."""
    try:
        from ..webp_file import GraphicWebpFileProvider
    except ImportError as err:
        raise err

    return GraphicWebpFileProvider(generator).graphic_webp_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        size=size,
        hue=hue,
        luminosity=luminosity,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ******************** XLSX **********************
# ************************************************


@overload
def create_inner_xlsx_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    data_columns: Optional[Dict[str, str]] = None,
    num_rows: int = 10,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_xlsx_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    data_columns: Optional[Dict[str, str]] = None,
    num_rows: int = 10,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_xlsx_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    data_columns: Optional[Dict[str, str]] = None,
    num_rows: int = 10,
    content: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner XLSX file."""
    try:
        from ..xlsx_file import XlsxFileProvider
    except ImportError as err:
        raise err

    return XlsxFileProvider(generator).xlsx_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        data_columns=data_columns,
        num_rows=num_rows,
        content=content,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* XML **********************
# ************************************************


@overload
def create_inner_xml_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    root_element: str = "root",
    row_element: str = "row",
    data_columns: Optional[Dict[str, str]] = None,
    num_rows: int = 10,
    content: Optional[str] = None,
    encoding: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_xml_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    root_element: str = "root",
    row_element: str = "row",
    data_columns: Optional[Dict[str, str]] = None,
    num_rows: int = 10,
    content: Optional[str] = None,
    encoding: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    **kwargs,
) -> StringValue: ...


def create_inner_xml_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    root_element: str = "root",
    row_element: str = "row",
    data_columns: Optional[Dict[str, str]] = None,
    num_rows: int = 10,
    content: Optional[str] = None,
    encoding: Optional[str] = None,
    format_func: Callable[
        [Union[Faker, Generator, Provider], str], str
    ] = DEFAULT_FORMAT_FUNC,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner XML file."""
    try:
        from ..xml_file import XmlFileProvider
    except ImportError as err:
        raise err

    return XmlFileProvider(generator).xml_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        root_element=root_element,
        row_element=row_element,
        data_columns=data_columns,
        num_rows=num_rows,
        content=content,
        encoding=encoding,
        format_func=format_func,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ********************* ZIP **********************
# ************************************************


@overload
def create_inner_zip_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    raw: bool = True,
    **kwargs,
) -> BytesValue: ...


@overload
def create_inner_zip_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    **kwargs,
) -> StringValue: ...


def create_inner_zip_file(
    storage: Optional[BaseStorage] = None,
    basename: Optional[str] = None,
    prefix: Optional[str] = None,
    generator: Optional[Union[Faker, Generator, Provider]] = None,
    options: Optional[Dict[str, Any]] = None,
    raw: bool = False,
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner ZIP file."""
    try:
        from ..zip_file import ZipFileProvider
    except ImportError as err:
        raise err
    return ZipFileProvider(generator).zip_file(
        storage=storage,
        basename=basename,
        prefix=prefix,
        options=options,
        raw=raw,
        **kwargs,
    )


# ************************************************
# ******************** OTHER *********************
# ************************************************


def fuzzy_choice_create_inner_file(
    func_choices: List[Tuple[Callable, Dict[str, Any]]],
    **kwargs,
) -> Union[BytesValue, StringValue]:
    """Create inner file from given list of function choices.

    :param func_choices: List of functions to choose from.
    :return: StringValue.

    Usage example:

    .. code-block:: python

        from faker import Faker
        from faker_file.providers.helpers.inner import (
            create_inner_docx_file,
            create_inner_epub_file,
            create_inner_txt_file,
            fuzzy_choice_create_inner_file,
        )
        from faker_file.storages.filesystem import FileSystemStorage

        FAKER = Faker()
        STORAGE = FileSystemStorage()

        kwargs = {"storage": STORAGE, "generator": FAKER}
        file = fuzzy_choice_create_inner_file(
            [
                (create_inner_docx_file, kwargs),
                (create_inner_epub_file, kwargs),
                (create_inner_txt_file, kwargs),
            ]
        )

    You could use it in archives to make a variety of different file types
    within the archive.

    .. code-block:: python

        from faker import Faker
        from faker_file.providers.helpers.inner import (
            create_inner_docx_file,
            create_inner_epub_file,
            create_inner_txt_file,
            fuzzy_choice_create_inner_file,
        )
        from faker_file.providers.zip_file import ZipFileProvider
        from faker_file.storages.filesystem import FileSystemStorage

        FAKER = Faker()
        FAKER.add_provider(ZipFileProvider)

        STORAGE = FileSystemStorage()

        kwargs = {"storage": STORAGE, "generator": FAKER}
        file = FAKER.zip_file(
            prefix="zzz_archive_",
            options={
                "count": 50,
                "create_inner_file_func": fuzzy_choice_create_inner_file,
                "create_inner_file_args": {
                    "func_choices": [
                        (create_inner_docx_file, kwargs),
                        (create_inner_epub_file, kwargs),
                        (create_inner_txt_file, kwargs),
                    ],
                },
                "directory": "zzz",
            }
        )
    """
    _func, _kwargs = choice(func_choices)
    return _func(**_kwargs)


def list_create_inner_file(
    func_list: List[Tuple[Callable, Dict[str, Any]]],
    **kwargs,
) -> List[Union[BytesValue, StringValue]]:
    """Generates multiple files based on the provided list of functions
    and arguments.

    :param func_list: List of tuples, each containing a function to generate a
        file and its arguments.
    :return: List of generated file names.

    Usage example:

    .. code-block:: python

        from faker import Faker
        from faker_file.providers.helpers.inner import (
            create_inner_docx_file,
            create_inner_xml_file,
            list_create_inner_file,
        )
        from faker_file.providers.zip_file import ZipFileProvider
        from faker_file.storages.filesystem import FileSystemStorage

        FAKER = Faker()
        FAKER.add_provider(ZipFileProvider)
        STORAGE = FileSystemStorage()

        kwargs = {"storage": STORAGE, "generator": FAKER}
        file = FAKER.zip_file(
            basename="alice-looking-through-the-glass",
            options={
                "create_inner_file_func": list_create_inner_file,
                "create_inner_file_args": {
                    "func_list": [
                        (create_inner_docx_file, {"basename": "doc"}),
                        (create_inner_xml_file, {"basename": "doc_metadata"}),
                        (create_inner_xml_file, {"basename": "doc_isbn"}),
                    ],
                },
            }
        )

    Note, that while all other inner functions return
    back `Union[BytesValue, StringValue]` value, `list_create_inner_file`
    returns back a `List[Union[BytesValue, StringValue]]` value.

    Notably, all inner functions were designed to support archives (such as
    ZIP, TAR and EML, but the list may grow in the future). If the inner
    function passed in the `create_inner_file_func` argument returns a List
    of `Union[BytesValue, StringValue]` values, the `option` argument is being
    ignored and generated files are simply limited to what has been passed
    in the `func_list` list of tuples.
    """
    created_files = []
    for func, kwargs in func_list:
        file = func(**kwargs)
        created_files.append(file)
    return created_files



import random
import socket

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "AutoIncPortInt",
    "AutoFreePortInt",
)


class AutoIncPortInt(int):
    """Automatically incremented integer value.

    Contains state of issued values. Starts from 2223. Every time
    initialized, value increases.

    Usage example:

        port = AutoInt()  # 2223
        port = AutoInt()  # 2224
        port = AutoInt()  # 2225

    For the rest, it behaves like a normal integer.

    For better integration, it's recommended to cast the value to `int`,
    like this:

        port = int(AutoInt())
    """

    # This is a class attribute that keeps the current state
    _next_value: int = 2223

    def __new__(cls, *args, **kwargs) -> "AutoIncPortInt":
        """Create a new instance of AutoInt with the next value."""
        # Use __new__ because int is immutable, so we need to set the value
        # before creating the instance.
        instance = super().__new__(cls, cls._next_value)
        # Increment the class attribute for the next instance
        cls._next_value += 1
        # Return the new instance
        return instance


class AutoFreePortInt(int):
    """Automatically and randomly picks a free port within a specified range.

    For instance:

        # Random free port between default range 2223 and 5000
        port = AutoFreePortInt()

        # Random free port between 3000 and 4000
        port = AutoFreePortInt(min_port=3000, max_port=4000)

    For the rest, it behaves like a normal integer.

    For better integration, it's recommended to cast the value to `int`,
    like this:

        port = int(AutoFreePortInt())
    """

    # Default port range
    DEFAULT_MIN_PORT: int = 2223
    DEFAULT_MAX_PORT: int = 5000

    @staticmethod
    def _is_port_free(host: str, port: int) -> bool:
        """Check if the given port is free."""
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            try:
                # Try to bind the socket to the port
                s.bind((host, port))
                return True
            except OSError:
                return False

    def __new__(
        cls,
        min_port: int = DEFAULT_MIN_PORT,
        max_port: int = DEFAULT_MAX_PORT,
        host: str = "localhost",
        *args,
        **kwargs,
    ) -> "AutoFreePortInt":
        """Create a new instance of `AutoFreePortInt`.

        With a free random port within the specified range.
        """
        # Validate the range
        if not (
            isinstance(min_port, int)
            and isinstance(max_port, int)
            and min_port <= max_port
        ):
            raise ValueError("Invalid port range specified")

        # Pick a random port within the specified range and check if it's free
        while True:
            port = random.randint(min_port, max_port)
            if cls._is_port_free(host, port):
                break

        # Use __new__ to create the instance with the free port as the value
        instance = super().__new__(cls, port)
        return instance



from faker import Faker
from faker_file.providers.txt_file import TxtFileProvider
from faker_file.storages.google_cloud_storage import GoogleCloudStorage

FAKER = Faker()
FAKER.add_provider(TxtFileProvider)

GC_STORAGE = GoogleCloudStorage(
    bucket_name="your-bucket-name",
    root_path="",
    rel_path="user/uploads",
)

# txt_file = FAKER.txt_file(storage=GC_STORAGE)



import unittest
from copy import deepcopy

from ..helpers import random_pop

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("HelpersTestCase",)


class HelpersTestCase(unittest.TestCase):
    """Test StringList test case."""

    def test_random_pop_empty_list(self: "HelpersTestCase") -> None:
        """Test `random_pop`."""
        my_list = []
        element = random_pop(my_list)
        self.assertIsNone(element)

    def test_random_pop(self: "HelpersTestCase") -> None:
        """Test `random_pop`."""
        my_list = [1, 2, 3, 4, 5]
        orig_my_list = deepcopy(my_list)
        while len(my_list):
            element = random_pop(my_list)
            self.assertIn(element, orig_my_list)
            self.assertNotIn(element, my_list)



from faker import Faker
from faker_file.providers.generic_file import GenericFileProvider

FAKER = Faker()
FAKER.add_provider(GenericFileProvider)

generic_file = FAKER.generic_file(
    content="<html><body><p>{{text}}</p></body></html>",
    extension="html",
)



from faker import Faker
from faker_file.providers.docx_file import DocxFileProvider

FAKER = Faker()
FAKER.add_provider(DocxFileProvider)

TEMPLATE = """
{{date}} {{city}}, {{country}}

Hello {{name}},

{{text}}

Address: {{address}}

Best regards,

{{name}}
{{address}}
{{phone_number}}
"""

docx_file = FAKER.docx_file(content=TEMPLATE)

print(docx_file)  # Sample value: 'tmp/tmpgdctmfbp.docx'
print(docx_file.data["content"])
# Sample value below:
#  2009-05-14 Pettyberg, Puerto Rico
#  Hello Lauren Williams,
#
#  Everyone bill I information. Put particularly note language support
#  green. Game free family probably case day vote.
#  Commercial especially game heart.
#
#  Address: 19017 Jennifer Drives
#  Jamesbury, MI 39121
#
#  Best regards,
#
#  Robin Jones
#  4650 Paul Extensions
#  Port Johnside, VI 78151
#  001-704-255-3093



from faker import Faker
from faker_file.providers.helpers.inner import create_inner_docx_file
from faker_file.providers.zip_file import ZipFileProvider

FAKER = Faker()
FAKER.add_provider(ZipFileProvider)

zip_file = FAKER.zip_file(
    prefix="zzz",
    options={
        "count": 3,
        "create_inner_file_func": create_inner_docx_file,
        "create_inner_file_args": {
            "prefix": "xxx_",
            "max_nb_chars": 1_024,
        },
        "directory": "yyy",
    },
)



from typing import (
    Any,
    Callable,
    Dict,
    List,
    Optional,
    Tuple,
    Union,
    get_type_hints,
)

from faker import Faker
from faker.generator import Generator
from faker.providers.python import Provider

from .helpers import wrap_text

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "BytesValue",
    "DEFAULT_FORMAT_FUNC",
    "DEFAULT_REL_PATH",
    "DynamicTemplate",
    "FileMixin",
    "StringList",
    "StringValue",
    "parse_format_func",
    "pystr_format_func",
    "returns_list",
)


DEFAULT_REL_PATH = "tmp"


class StringValue(str):
    __slots__ = ("data",)

    data: Dict[str, Any]

    def __new__(cls, value, *args, **kwargs):
        obj = str.__new__(cls, value)
        obj.data = {}
        return obj


class BytesValue(bytes):
    data: Dict[str, Any]

    def __new__(cls, value, *args, **kwargs):
        obj = bytes.__new__(cls, value)
        obj.data = {}
        return obj


def parse_format_func(
    generator: Union[Faker, Generator, Provider], content: str
) -> str:
    return generator.parse(content)


def pystr_format_func(
    generator: Union[Faker, Generator, Provider], content: str
) -> str:
    return generator.pystr_format(content)


DEFAULT_FORMAT_FUNC = parse_format_func


class FileMixin:
    """File mixin."""

    numerify: Callable
    random_element: Callable
    formats: List[str]
    generator: Union[Faker, Generator, Provider]
    extension: str  # Desired file extension.

    def _generate_text_content(
        self,
        max_nb_chars: int,
        wrap_chars_after: Optional[int] = None,
        content: Optional[str] = None,
        format_func: Callable[
            [Union[Faker, Generator, Provider], str], str
        ] = DEFAULT_FORMAT_FUNC,
    ) -> str:
        """Generate text content.

        :param max_nb_chars: Max number of chars.
        :param wrap_chars_after: If given, the output string would be separated
             by line breaks after the given position.
        :return: String with random chars.
        """
        if self.generator is None:
            self.generator = Faker()

        # Specific
        if content is None:
            content = self.generator.text(max_nb_chars)
        else:
            content = format_func(self.generator, content)

        if wrap_chars_after:
            content = wrap_text(content, wrap_chars_after)

        return content


class DynamicTemplate:
    """Dynamic template."""

    def __init__(
        self, content_modifiers: List[Tuple[Callable, Dict[str, Any]]]
    ):
        self.content_modifiers = content_modifiers


class StringList:
    """String list.

    Usage example:

    .. code-block:: python
        :name: test_rst_string_list

        my_string = StringList(separator="\r\n")
        my_string += "grape"
        my_string += "peaches"
        print(my_string)
    """

    def __init__(
        self: "StringList",
        strings: Optional[List[str]] = None,
        separator: str = " ",
    ) -> None:
        self.strings = strings if strings is not None else []
        self.separator = separator

    def __repr__(self: "StringList") -> str:
        return self.__str__()

    def __str__(self: "StringList") -> str:
        return self.separator.join(self.strings)

    def __iadd__(self: "StringList", value: str) -> "StringList":
        self.add_string(value)
        return self

    def add_string(self: "StringList", value: str) -> None:
        self.strings.append(value)

    def remove_string(self: "StringList", value: str) -> None:
        if value in self.strings:
            self.strings.remove(value)


def returns_list(func: Callable) -> bool:
    """Checks if callable returns a list of Union[BytesValue, StringValue].

    Returns True if it's a List. Returns False otherwise.
    """
    try:
        return_type = get_type_hints(func).get("return", None)
    except Exception:
        return False

    if not return_type:
        return False

    return_origin = getattr(return_type, "__origin__", None)
    if return_origin is list or return_origin is List:
        # If it's a list, check the type of its elements
        element_type = getattr(return_type, "__args__", [None])[0]
        element_origin = getattr(element_type, "__origin__", None)
        if element_origin is Union:
            if set(getattr(element_type, "__args__", [])) == {
                BytesValue,
                StringValue,
            }:
                return True

    return False



from faker import Faker
from faker_file.providers.helpers.inner import create_inner_docx_file
from faker_file.providers.zip_file import ZipFileProvider

FAKER = Faker()
FAKER.add_provider(ZipFileProvider)

TEMPLATE = "Hey {{name}},\n{{text}},\nBest regards\n{{name}}"

zip_file = FAKER.zip_file(
    options={
        "count": 9,
        "create_inner_file_func": create_inner_docx_file,
        "create_inner_file_args": {
            "content": TEMPLATE,
        },
    }
)



import logging
import textwrap
from io import BytesIO
from pathlib import Path
from typing import Any, Dict, List, Type, Union

from faker import Faker
from faker.generator import Generator
from faker.providers.python import Provider
from PIL import Image, ImageDraw, ImageFont

from ...base import DynamicTemplate
from ...constants import DEFAULT_FILE_ENCODING
from ..base.image_generator import BaseImageGenerator

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("PilImageGenerator",)

LOGGER = logging.getLogger(__name__)


class PilImageGenerator(BaseImageGenerator):
    """PIL image generator.

    Usage example:

    .. code-block:: python

        from faker import Faker
        from faker_file.providers.png_file import PngFileProvider
        from faker_file.providers.image.pil_generator import PilImageGenerator

        FAKER = Faker()
        FAKER.add_provider(PngFileProvider)

        file = FAKER.png_file(
            image_generator_cls=PilImageGenerator,
        )

    With options:

    .. code-block:: python

        file = FAKER.png_file(
            image_generator_cls=PilImageGenerator,
            image_generator_kwargs={
                "spacing": 6,
            },
            wrap_chars_after=119,
        )

    With dynamic content:

    .. code-block:: python

        from faker import Faker
        from faker_file.base import DynamicTemplate
        from faker_file.contrib.image.pil_snippets import *
        from faker_file.providers.image.pil_generator import PilImageGenerator
        from faker_file.providers.png_file import PngFileProvider

        FAKER = Faker()
        FAKER.add_provider(PngFileProvider)

        file = FAKER.png_file(
            image_generator_cls=PilImageGenerator,
            content=DynamicTemplate(
                [
                    (add_h1_heading, {}),
                    (add_paragraph, {"max_nb_chars": 500}),
                    (add_paragraph, {"max_nb_chars": 500}),
                    (add_paragraph, {"max_nb_chars": 500}),
                    (add_paragraph, {"max_nb_chars": 500}),
                ]
            )
        )

        file = FAKER.png_file(
            image_generator_cls=PilImageGenerator,
            content=DynamicTemplate(
                [
                    (add_h1_heading, {}),
                    (add_paragraph, {}),
                    (add_picture, {}),
                    (add_paragraph, {}),
                    (add_picture, {}),
                    (add_paragraph, {}),
                    (add_picture, {}),
                    (add_paragraph, {}),
                ]
            )
        )

        file = FAKER.png_file(
            image_generator_cls=PilImageGenerator,
            content=DynamicTemplate(
                [
                    (add_h1_heading, {}),
                    (add_picture, {}),
                    (add_paragraph, {"max_nb_chars": 500}),
                    (add_picture, {}),
                    (add_paragraph, {"max_nb_chars": 500}),
                    (add_picture, {}),
                    (add_paragraph, {"max_nb_chars": 500}),
                    (add_picture, {}),
                    (add_paragraph, {"max_nb_chars": 500}),
                ]
            )
        )

        file = FAKER.png_file(
            image_generator_cls=PilImageGenerator,
            content=DynamicTemplate(
                [
                    (add_h1_heading, {}),
                    (add_picture, {}),
                    (add_paragraph, {"max_nb_chars": 500}),
                    (add_table, {"rows": 5, "cols": 4}),
                ]
            )
        )

        file = FAKER.png_file(
            image_generator_cls=PilImageGenerator,
            content=DynamicTemplate(
                [
                    (add_h1_heading, {"margin": (2, 2)}),
                    (add_picture, {"margin": (2, 2)}),
                    (add_paragraph, {"max_nb_chars": 500, "margin": (2, 2)}),
                    (add_picture, {"margin": (2, 2)}),
                    (add_paragraph, {"max_nb_chars": 500, "margin": (2, 2)}),
                    (add_picture, {"margin": (2, 2)}),
                    (add_paragraph, {"max_nb_chars": 500, "margin": (2, 2)}),
                    (add_picture, {"margin": (2, 2)}),
                    (add_paragraph, {"max_nb_chars": 500, "margin": (2, 2)}),
                ]
            )
        )
    """

    encoding: str = DEFAULT_FILE_ENCODING
    font: str = str(Path("Pillow") / "Tests" / "fonts" / "DejaVuSans.ttf")
    font_size: int = 12
    page_width: int = 794  # A4 size at 96 DPI
    page_height: int = 1123  # A4 size at 96 DPI
    line_height: int = 14
    spacing: int = 6

    def __init__(self: "PilImageGenerator", **kwargs) -> None:
        super().__init__(**kwargs)
        self.pages = []
        self.img = None
        self.draw = None
        self.image_mode = "RGB"

    @classmethod
    def find_max_fit_for_multi_line_text(
        cls: Type["PilImageGenerator"],
        draw: ImageDraw,
        lines: List[str],
        font: ImageFont,
        max_width: int,
    ):
        # Find the longest line
        longest_line = str(max(lines, key=len))
        return cls.find_max_fit_for_single_line_text(
            draw, longest_line, font, max_width
        )

    @classmethod
    def find_max_fit_for_single_line_text(
        cls: Type["PilImageGenerator"],
        draw: "ImageDraw",
        text: str,
        font: ImageFont,
        max_width: int,
    ) -> int:
        low, high = 0, len(text)
        while low < high:
            mid = (high + low) // 2
            text_width, _ = draw.textsize(text[:mid], font=font)

            if text_width > max_width:
                high = mid
            else:
                low = mid + 1

        return low - 1

    def handle_kwargs(self: "PilImageGenerator", **kwargs) -> None:
        """Handle kwargs."""
        if "encoding" in kwargs:
            self.encoding = kwargs["encoding"]
        if "font_size" in kwargs:
            self.font_size = kwargs["font_size"]
        if "page_width" in kwargs:
            self.page_width = kwargs["page_width"]
        if "page_height" in kwargs:
            self.page_height = kwargs["page_height"]
        if "line_height" in kwargs:
            self.line_height = kwargs["line_height"]
        if "spacing" in kwargs:
            self.spacing = kwargs["spacing"]
        if "image_mode" in kwargs:
            self.image_mode = kwargs["image_mode"]

    def create_image_instance(self, height: Union[int, None] = None) -> Image:
        return Image.new(
            self.image_mode,
            (self.page_width, height or self.page_height),
            (255, 255, 255),
        )

    def start_new_page(self):
        self.img = self.create_image_instance()
        self.draw = ImageDraw.Draw(self.img)

    def save_and_start_new_page(self):
        self.pages.append(self.img.copy())
        self.start_new_page()

    def combine_images_vertically(self):
        # Calculate total width and height
        total_width = max(image.width for image in self.pages)
        total_height = sum(image.height for image in self.pages)

        # Create a new, white canvas to paste images onto
        new_image = Image.new("RGB", (total_width, total_height), "white")

        # Paste each image
        y_offset = 0
        for image in self.pages:
            new_image.paste(image, (0, y_offset))
            y_offset += image.height

        return new_image

    def generate(
        self: "PilImageGenerator",
        content: str,
        data: Dict[str, Any],
        provider: Union[Faker, Generator, Provider],
        **kwargs,
    ) -> bytes:
        """Generate image."""
        position = (0, 0)
        if isinstance(content, DynamicTemplate):
            self.start_new_page()
            for counter, (ct_modifier, ct_modifier_kwargs) in enumerate(
                content.content_modifiers
            ):
                ct_modifier_kwargs["position"] = position
                # LOGGER.debug(f"ct_modifier_kwargs: {ct_modifier_kwargs}")
                add_page, position = ct_modifier(
                    provider,
                    self,
                    data,
                    counter,
                    **ct_modifier_kwargs,
                )

            self.pages.append(self.img.copy())  # Add as a new page
        else:
            self.img = self.create_image_instance()
            self.draw = ImageDraw.Draw(self.img)
            font = ImageFont.truetype(self.font, self.font_size)

            # The `content_specs` is a dictionary that holds two keys:
            # `max_nb_chars` and `wrap_chars_after`. Those are the same values
            # passed to the `PdfFileProvider`.
            content_specs = kwargs.get("content_specs", {})
            lines = content.split("\n")
            line_max_num_chars = self.find_max_fit_for_multi_line_text(
                self.draw,
                lines,
                font,
                self.page_width,
            )
            wrap_chars_after = content_specs.get("wrap_chars_after")
            if (
                not wrap_chars_after
                or wrap_chars_after
                and (wrap_chars_after > line_max_num_chars)
            ):
                lines = textwrap.wrap(content, line_max_num_chars)

            y_text = 0
            for counter, line in enumerate(lines):
                text_width, text_height = self.draw.textsize(
                    line, font=font, spacing=self.spacing
                )
                if y_text + text_height > self.page_height:
                    self.save_and_start_new_page()
                    y_text = 0

                self.draw.text(
                    (0, y_text),
                    line,
                    fill=(0, 0, 0),
                    spacing=self.spacing,
                    font=font,
                )
                y_text += text_height + self.line_height

            self.pages.append(self.img.copy())  # Add as a new page

        buffer = BytesIO()
        # Combine images together vertically
        combined_image = self.combine_images_vertically()
        combined_image.save(
            buffer,
            resolution=100.0,
            quality=95,
            format=provider.image_format,
        )

        return buffer.getvalue()



import logging
from threading import Lock
from typing import Optional, Set, Union

from .base import StringValue

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "FILE_REGISTRY",
    "FileRegistry",
)


LOGGER = logging.getLogger(__name__)


class FileRegistry:
    """Stores list `StringValue` instances.

    .. code-block:: python

        from faker import Faker
        from faker_file.providers.txt_file import TxtFileProvider

        FAKER = Faker()
        FAKER.add_provider(TxtFileProvider)

        txt_file_1 = FAKER.txt_file()
        txt_file_2 = FAKER.txt_file()
        ...
        txt_file_n = FAKER.txt_file()

        # The FileRegistry._registry would then contain this:
        {
            txt_file_1,
            txt_file_2,
            ...,
            txt_file_n,
        }
    """

    def __init__(self):
        self._registry: Set[StringValue] = set()
        self._lock = Lock()

    def add(self, string_value: StringValue) -> None:
        with self._lock:
            self._registry.add(string_value)

    def remove(self, string_value: Union[StringValue, str]) -> bool:
        if not isinstance(string_value, StringValue):
            string_value = self.search(string_value)

        if not string_value:
            return False

        with self._lock:
            # No error if the element doesn't exist
            self._registry.discard(string_value)
            try:
                string_value.data["storage"].unlink(
                    string_value.data["filename"]
                )
                return True
            except Exception as e:
                LOGGER.error(
                    f"Failed to unlink file "
                    f"{string_value.data['filename']}: {e}"
                )
            return False

    def search(self, value: str) -> Optional[StringValue]:
        with self._lock:
            for string_value in self._registry:
                if string_value == value:
                    return string_value
        return None

    def clean_up(self) -> None:
        with self._lock:
            while self._registry:
                file = self._registry.pop()
                try:
                    file.data["storage"].unlink(file.data["filename"])
                except Exception as err:
                    LOGGER.error(
                        f"Failed to unlink file {file.data['filename']}: {err}"
                    )


FILE_REGISTRY = FileRegistry()



import unittest

from PIL import Image, ImageDraw

from ..providers.image.augment import solarize

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("TestSolarizeFunction",)


class TestSolarizeFunction(unittest.TestCase):
    """Test `solarize` function."""

    def create_rgba_image(self):
        """Create a sample RGBA image."""
        img = Image.new("RGBA", (100, 100), (150, 60, 90, 255))
        draw = ImageDraw.Draw(img)
        draw.rectangle((25, 25, 75, 75), fill=(60, 170, 210, 200))
        return img

    def create_rgb_image(self):
        """Create a sample RGB image."""
        img = Image.new("RGB", (100, 100), (150, 60, 90))
        draw = ImageDraw.Draw(img)
        draw.rectangle((25, 25, 75, 75), fill=(60, 170, 210))
        return img

    def test_solarize_rgba(self):
        img = self.create_rgba_image()
        solarized_img = solarize(img)

        # Assuming values above 128 are inverted, check some pixel values
        self.assertEqual(solarized_img.getpixel((10, 10)), (105, 60, 90, 255))
        self.assertEqual(
            solarized_img.getpixel((50, 50)), (60, 85, 45, 200)
        )  # Alpha should remain unaffected

    def test_solarize_rgb(self):
        img = self.create_rgb_image()
        solarized_img = solarize(img)

        # Assuming values above 128 are inverted, check some pixel values
        self.assertEqual(solarized_img.getpixel((10, 10)), (105, 60, 90))
        self.assertEqual(solarized_img.getpixel((50, 50)), (60, 85, 45))



import asyncio
import logging
from io import BytesIO

import edge_tts

from ...base.mp3_generator import BaseMp3Generator

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("EdgeTtsMp3Generator",)

DEFAULT_VOICE = "en-GB-SoniaNeural"
LOGGER = logging.getLogger(__name__)


class EdgeTtsMp3Generator(BaseMp3Generator):
    """Edge Text-to-Speech generator.

    Usage example:

    .. code-block:: python

        from faker import Faker
        from faker_file.providers.mp3_file import Mp3FileProvider
        from faker_file.providers.mp3_file.generators import edge_tts_generator

        FAKER = Faker()
        FAKER.add_provider(Mp3FileProvider)

        file = FAKER.mp3_file(
            mp3_generator_cls=edge_tts_generator.EdgeTtsMp3Generator
        )
    """

    voice: str = DEFAULT_VOICE

    def handle_kwargs(self: "EdgeTtsMp3Generator", **kwargs) -> None:
        if "voice" in kwargs:
            self.voice = kwargs["voice"]

    def generate(self: "EdgeTtsMp3Generator", **kwargs) -> bytes:
        """Generate MP3."""
        with BytesIO() as _fake_file:

            async def _generate():
                """Async generate.

                edge-tts is an async library. We can't use it in non-async
                context. Therefore, this trick.
                """
                async for chunk in edge_tts.Communicate(
                    self.content, self.voice
                ).stream():
                    if chunk["type"] == "audio":
                        _fake_file.write(chunk["data"])
                    elif chunk["type"] == "WordBoundary":
                        LOGGER.debug(f"WordBoundary: {chunk}")

            # Run the async function
            loop = asyncio.get_event_loop_policy().new_event_loop()
            loop.run_until_complete(_generate())
            # Return result
            loop.close()
            return _fake_file.getvalue()



"""create upload table

Revision ID: 2695cb77cdf2
Revises:
Create Date: 2022-12-06 23:46:09.000000

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "2695cb77cdf2"
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    op.create_table(
        "upload",
        sa.Column("id", sa.Integer, primary_key=True),
        sa.Column("name", sa.String(64), nullable=False),
        sa.Column("description", sa.Text(), nullable=True),
        sa.Column("file", sa.String(255), nullable=False),
    )


def downgrade():
    op.drop_table("upload")



from typing import Optional, Union, overload

from faker import Faker
from faker.providers import BaseProvider

from ..base import BytesValue, FileMixin, StringValue
from ..registry import FILE_REGISTRY
from ..storages.base import BaseStorage
from ..storages.filesystem import FileSystemStorage

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("BinFileProvider",)


class BinFileProvider(BaseProvider, FileMixin):
    """BIN file provider.

    Usage example:

    .. code-block:: python

        from faker import Faker
        from faker_file.providers.bin_file import BinFileProvider

        FAKER = Faker()
        FAKER.add_provider(BinFileProvider)

        file = FAKER.bin_file()

    Usage example with options:

    .. code-block:: python

        file = FAKER.bin_file(
            prefix="zzz",
            length=1024**2,
        )

    Usage example with `FileSystemStorage` storage (for `Django`):

    .. code-block:: python

        from django.conf import settings
        from faker_file.storages.filesystem import FileSystemStorage

        file = FAKER.bin_file(
            storage=FileSystemStorage(
                root_path=settings.MEDIA_ROOT,
                rel_path="tmp",
            ),
            prefix="zzz",
            length=1024**2,
        )

    Usage example with AWS S3 storage:

    .. code-block:: python

        from faker_file.storages.aws_s3 import AWSS3Storage

        file = FAKER.bin_file(
            storage=AWSS3Storage(bucket_name="My-test-bucket"),
            prefix="zzz",
            length=1024**2,
        )
    """

    extension: str = "bin"

    @overload
    def bin_file(
        self: "BinFileProvider",
        storage: Optional[BaseStorage] = None,
        basename: Optional[str] = None,
        prefix: Optional[str] = None,
        length: int = (1 * 1024 * 1024),
        content: Optional[bytes] = None,
        raw: bool = True,
        **kwargs,
    ) -> BytesValue: ...

    @overload
    def bin_file(
        self: "BinFileProvider",
        storage: Optional[BaseStorage] = None,
        basename: Optional[str] = None,
        prefix: Optional[str] = None,
        length: int = (1 * 1024 * 1024),
        content: Optional[bytes] = None,
        **kwargs,
    ) -> StringValue: ...

    def bin_file(
        self: "BinFileProvider",
        storage: Optional[BaseStorage] = None,
        basename: Optional[str] = None,
        prefix: Optional[str] = None,
        length: int = (1 * 1024 * 1024),
        content: Optional[bytes] = None,
        raw: bool = False,
        **kwargs,
    ) -> Union[BytesValue, StringValue]:
        """Generate a BIN file with random bytes.

        :param storage: Storage class. Defaults to `FileSystemStorage`.
        :param basename: File basename (without extension).
        :param prefix: File name prefix.
        :param length:
        :param content: File content. If given, used as is.
        :param raw: If set to True, return `BytesValue` (binary content of
            the file). Otherwise, return `StringValue` (path to the saved
            file).
        :return: Relative path (from root directory) of the generated file
            or raw content of the file.
        """
        # Generic
        if storage is None:
            storage = FileSystemStorage()

        filename = storage.generate_filename(
            extension=self.extension,
            prefix=prefix,
            basename=basename,
        )

        if self.generator is None:
            self.generator = Faker()

        # Specific
        if content is None:
            content = self.generator.binary(length=length)

        data = {"content": content, "filename": filename, "storage": storage}

        if raw:
            raw_content = BytesValue(content)
            raw_content.data = data
            return raw_content

        storage.write_bytes(filename, content)

        # Generic
        file_name = StringValue(storage.relpath(filename))
        file_name.data = data
        FILE_REGISTRY.add(file_name)
        return file_name



from typing import Optional, Sequence, Tuple, Union, overload

from faker import Faker

from ...base import BytesValue, FileMixin, StringValue
from ...registry import FILE_REGISTRY
from ...storages.base import BaseStorage
from ...storages.filesystem import FileSystemStorage

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("GraphicImageMixin",)


class GraphicImageMixin(FileMixin):
    """Graphic image mixin."""

    @overload
    def _image_file(
        self: "GraphicImageMixin",
        image_format: str,
        storage: Optional[BaseStorage] = None,
        basename: Optional[str] = None,
        prefix: Optional[str] = None,
        size: Tuple[int, int] = (256, 256),
        hue: Union[int, Sequence[int], str, None] = None,
        luminosity: Optional[str] = None,
        raw: bool = True,
        **kwargs,
    ) -> BytesValue: ...

    @overload
    def _image_file(
        self: "GraphicImageMixin",
        image_format: str,
        storage: Optional[BaseStorage] = None,
        basename: Optional[str] = None,
        prefix: Optional[str] = None,
        size: Tuple[int, int] = (256, 256),
        hue: Union[int, Sequence[int], str, None] = None,
        luminosity: Optional[str] = None,
        **kwargs,
    ) -> StringValue: ...

    def _image_file(
        self: "GraphicImageMixin",
        image_format: str,
        storage: Optional[BaseStorage] = None,
        basename: Optional[str] = None,
        prefix: Optional[str] = None,
        size: Tuple[int, int] = (256, 256),
        hue: Union[int, Sequence[int], str, None] = None,
        luminosity: Optional[str] = None,
        raw: bool = False,
        **kwargs,
    ) -> Union[BytesValue, StringValue]:
        """Generate an image file with random text.

        :param image_format: Image format. Can be any valid format to the
            underlying library like "tiff", "jpeg", "pdf" or "png" (default).
            Note that some formats need present system libraries prior to
            building the Python Image Library. Refer to
            ://pillow.readthedocs.io/en/stable/handbook/image-file-formats.html
            for details.
        :param storage: Storage. Defaults to `FileSystemStorage`.
        :param basename: File basename (without extension).
        :param prefix: File name prefix.
        :param size: Image size in pixels.
        :param hue: Read more about
            ://faker.readthedocs.io/en/dev/providers/faker.providers.color.html
        :param luminosity: If given, the output string would be separated
             by line breaks after the given position.
        :param raw: If set to True, return `BytesValue` (binary content of
            the file). Otherwise, return `StringValue` (path to the saved
            file).
        :return: Relative path (from root directory) of the generated file
            or raw content of the file.
        """
        # Generic
        if storage is None:
            storage = FileSystemStorage()

        if self.generator is None:
            self.generator = Faker()

        filename = storage.generate_filename(
            extension=self.extension,
            prefix=prefix,
            basename=basename,
        )

        data = {"filename": filename, "storage": storage}

        buffer = self.generator.image(
            size=size,
            image_format=image_format,
            hue=hue,
            luminosity=luminosity,
        )

        if raw:
            raw_content = BytesValue(buffer)
            raw_content.data = data
            return raw_content

        storage.write_bytes(filename, buffer)

        # Generic
        file_name = StringValue(storage.relpath(filename))
        file_name.data = data
        FILE_REGISTRY.add(file_name)
        return file_name



from django.db import models


class Upload(models.Model):
    """Upload model."""

    name = models.CharField(max_length=255, unique=True)
    description = models.TextField(null=True, blank=True)

    # File
    file = models.FileField(null=True)

    class Meta:
        app_label = "uploads"  # Ignore this line when copying
        verbose_name = "Upload"
        verbose_name_plural = "Upload"

    def __str__(self):
        return self.name



from django.conf import settings
from django.core.files.storage import default_storage
from faker_file.storages.aws_s3 import AWSS3Storage
from faker_file.storages.filesystem import FileSystemStorage
from storages.backends.s3boto3 import S3Boto3Storage

# Faker doesn't know anything about Django. That's why, if we want to
# support remote storages, we need to manually check which file storage
# backend is used. If `Boto3` storage backend (of the `django-storages`
# package) is used we use the correspondent `AWSS3Storage` class of the
# `faker-file`.
# Otherwise, fall back to native file system storage (`FileSystemStorage`)
# of the `faker-file`.
if isinstance(default_storage, S3Boto3Storage):
    STORAGE = AWSS3Storage(
        bucket_name=settings.AWS_STORAGE_BUCKET_NAME,
        credentials={
            "key_id": settings.AWS_ACCESS_KEY_ID,
            "key_secret": settings.AWS_SECRET_ACCESS_KEY,
        },
        root_path="",
        rel_path="tmp",
    )
else:
    STORAGE = FileSystemStorage(
        root_path=settings.MEDIA_ROOT,
        rel_path="tmp",
    )



from typing import Any, Callable, Dict

from django.conf import settings
from django.core.files.storage import default_storage
from django.test import TestCase
from faker import Faker
from parametrize import parametrize
from storages.backends.s3boto3 import S3Boto3Storage

import factories

from ..registry import FILE_REGISTRY
from ..storages.aws_s3 import AWSS3Storage
from ..storages.filesystem import FileSystemStorage

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("DjangoIntegrationTestCase",)


# Faker doesn't know anything about Django. That's why, if we want to support
# remote storages, we need to manually check which file storage backend is
# used. If `Boto3` storage backend (of the `django-storages` package) is used
# we use the correspondent `AWSS3Storage` class of the `faker-file`.
# Otherwise, fall back to native file system storage (`FileSystemStorage`) of
# the `faker-file`.
if isinstance(default_storage, S3Boto3Storage):
    STORAGE = AWSS3Storage(
        bucket_name=settings.AWS_STORAGE_BUCKET_NAME,
        credentials={
            "key_id": settings.AWS_ACCESS_KEY_ID,
            "key_secret": settings.AWS_SECRET_ACCESS_KEY,
        },
        rel_path="tmp",
    )
else:
    STORAGE = FileSystemStorage(root_path=settings.MEDIA_ROOT, rel_path="tmp")


class DjangoIntegrationTestCase(TestCase):
    """Django integration test case."""

    FAKER: Faker

    def tearDown(self, *args, **kwargs) -> None:
        super().tearDown(*args, **kwargs)
        FILE_REGISTRY.clean_up()

    @parametrize(
        "factory, kwargs",
        [
            (factories.UploadFactory, {}),
            (factories.UploadFactory, {"random_file": True}),
            (factories.UploadFactory, {"pdf_file": True}),
            (factories.UploadFactory, {"pptx_file": True}),
            (factories.UploadFactory, {"txt_file": True}),
            (factories.UploadFactory, {"zip_file": True}),
        ],
    )
    def test_file(
        self: "DjangoIntegrationTestCase",
        factory: Callable,
        kwargs: Dict[str, Any],
    ) -> None:
        """Test file."""
        _upload = factory(**kwargs)
        if kwargs:
            self.assertTrue(STORAGE.exists(_upload.file.name))



from faker import Faker
from faker_file.providers.eml_file import EmlFileProvider

FAKER = Faker()
FAKER.add_provider(EmlFileProvider)

eml_file = FAKER.eml_file(
    options={"create_inner_file_args": {"content": "Lorem ipsum"}}
)



from faker import Faker
from faker_file.providers.txt_file import TxtFileProvider
from faker_file.registry import FILE_REGISTRY  # Import instance at once

FAKER = Faker()
FAKER.add_provider(TxtFileProvider)

# Create a file to remove
txt_file = FAKER.txt_file()

# We assume that there's an initialized `txt_file` instance to remove.
FILE_REGISTRY.remove(txt_file)  # Where file is an instance of ``StringValue``



from faker import Faker
from faker_file.providers.docx_file import DocxFileProvider

FAKER = Faker()
FAKER.add_provider(DocxFileProvider)

docx_file = FAKER.docx_file(
    prefix="zzz",
    max_nb_chars=1_024,
    wrap_chars_after=80,
)



from faker import Faker
from faker_file.providers.webp_file import GraphicWebpFileProvider

FAKER = Faker()
FAKER.add_provider(GraphicWebpFileProvider)

webp_file = FAKER.graphic_webp_file(size=(800, 800))



import os
import tempfile

import ffmpeg
from faker_file.providers.base.mp3_generator import BaseMp3Generator
from speak2mary import MaryTTS

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("MaryTtsMp3Generator",)


DEFAULT_LOCALE = "en_GB"
DEFAULT_VOICE = "dfki-spike-hsmm"


class MaryTtsMp3Generator(BaseMp3Generator):
    """MaryTTS Text-to-Speech generator.

    Usage example:

        from faker import Faker
        from faker_file.providers.mp3_file import Mp3FileProvider
        from marytts_mp3_generator import (
            MaryTtsMp3Generator,
        )

        FAKER = Faker()

        file = Mp3FileProvider(FAKER).mp3_file(
            mp3_generator_cls=MaryTtsMp3Generator,
            mp3_generator_kwargs={},
        )
    """

    locale: str = DEFAULT_LOCALE
    voice: str = DEFAULT_VOICE

    def handle_kwargs(self: "MaryTtsMp3Generator", **kwargs) -> None:
        if "locale" in kwargs:
            self.locale = kwargs["locale"]
        if "voice" in kwargs:
            self.voice = kwargs["voice"]

    def generate(self: "MaryTtsMp3Generator") -> bytes:
        """Generate MP3."""
        # Initialize with args
        mary_tts = MaryTTS(locale=self.locale, voice=self.voice)

        # Generate temporary filename for WAV file
        filename = tempfile.NamedTemporaryFile(
            prefix="_merytts_", suffix=".wav"
        ).name

        # Write WAV file
        with open(filename, "wb") as file:
            file.write(mary_tts.speak(self.content))

        # Convert WAV to MP3
        ffmpeg.input(filename).output(filename + ".mp3").run()

        with open(filename + ".mp3", "rb") as _fake_file:
            return_value = _fake_file.read()

        # Clean up temporary files
        os.remove(filename)
        os.remove(filename + ".mp3")

        return return_value



from django.conf import settings
from factory import Faker, Trait
from factory.django import DjangoModelFactory

# Import all providers we want to use
from faker_file.providers.bin_file import BinFileProvider
from faker_file.providers.csv_file import CsvFileProvider
from faker_file.providers.docx_file import DocxFileProvider
from faker_file.providers.eml_file import EmlFileProvider
from faker_file.providers.epub_file import EpubFileProvider
from faker_file.providers.ico_file import IcoFileProvider
from faker_file.providers.jpeg_file import JpegFileProvider
from faker_file.providers.mp3_file import Mp3FileProvider
from faker_file.providers.ods_file import OdsFileProvider
from faker_file.providers.odt_file import OdtFileProvider
from faker_file.providers.pdf_file import PdfFileProvider
from faker_file.providers.png_file import PngFileProvider
from faker_file.providers.pptx_file import PptxFileProvider
from faker_file.providers.rtf_file import RtfFileProvider
from faker_file.providers.svg_file import SvgFileProvider
from faker_file.providers.txt_file import TxtFileProvider
from faker_file.providers.webp_file import WebpFileProvider
from faker_file.providers.xlsx_file import XlsxFileProvider
from faker_file.providers.zip_file import ZipFileProvider

# Import file storage, because we need to customize things in order for it
# to work with Django.
from faker_file.storages.filesystem import FileSystemStorage

from upload.models import Upload

# Add all providers we want to use
Faker.add_provider(BinFileProvider)
Faker.add_provider(CsvFileProvider)
Faker.add_provider(DocxFileProvider)
Faker.add_provider(EmlFileProvider)
Faker.add_provider(EpubFileProvider)
Faker.add_provider(IcoFileProvider)
Faker.add_provider(JpegFileProvider)
Faker.add_provider(Mp3FileProvider)
Faker.add_provider(OdsFileProvider)
Faker.add_provider(OdtFileProvider)
Faker.add_provider(PdfFileProvider)
Faker.add_provider(PngFileProvider)
Faker.add_provider(PptxFileProvider)
Faker.add_provider(RtfFileProvider)
Faker.add_provider(SvgFileProvider)
Faker.add_provider(TxtFileProvider)
Faker.add_provider(WebpFileProvider)
Faker.add_provider(XlsxFileProvider)
Faker.add_provider(ZipFileProvider)

# Define a file storage. When working with Django and FileSystemStorage
# you need to set the value of `root_path` argument to
# `settings.MEDIA_ROOT`.
STORAGE = FileSystemStorage(root_path=settings.MEDIA_ROOT, rel_path="tmp")


class UploadFactory(DjangoModelFactory):
    """Upload factory."""

    name = Faker("text", max_nb_chars=100)
    description = Faker("text", max_nb_chars=1000)

    class Meta:
        model = Upload

    class Params:
        bin_file = Trait(file=Faker("bin_file", storage=STORAGE))
        csv_file = Trait(file=Faker("csv_file", storage=STORAGE))
        docx_file = Trait(file=Faker("docx_file", storage=STORAGE))
        eml_file = Trait(file=Faker("eml_file", storage=STORAGE))
        epub_file = Trait(file=Faker("epub_file", storage=STORAGE))
        ico_file = Trait(file=Faker("ico_file", storage=STORAGE))
        jpeg_file = Trait(file=Faker("jpeg_file", storage=STORAGE))
        mp3_file = Trait(file=Faker("mp3_file", storage=STORAGE))
        ods_file = Trait(file=Faker("ods_file", storage=STORAGE))
        odt_file = Trait(file=Faker("odt_file", storage=STORAGE))
        pdf_file = Trait(file=Faker("pdf_file", storage=STORAGE))
        png_file = Trait(file=Faker("png_file", storage=STORAGE))
        pptx_file = Trait(file=Faker("pptx_file", storage=STORAGE))
        rtf_file = Trait(file=Faker("rtf_file", storage=STORAGE))
        svg_file = Trait(file=Faker("svg_file", storage=STORAGE))
        txt_file = Trait(file=Faker("txt_file", storage=STORAGE))
        webp_file = Trait(file=Faker("webp_file", storage=STORAGE))
        xlsx_file = Trait(file=Faker("xlsx_file", storage=STORAGE))
        zip_file = Trait(file=Faker("zip_file", storage=STORAGE))


# Usage examples
UploadFactory(bin_file=True)  # Upload with BIN file
UploadFactory(docx_file=True)  # Upload with DOCX file
UploadFactory(jpeg_file=True)  # Upload with JPEG file
UploadFactory(zip_file=True)  # Upload with ZIP file



import os
from pathlib import Path
from random import choice
from typing import Optional, Union, overload

from faker.providers import BaseProvider

from ..base import BytesValue, FileMixin, StringValue
from ..registry import FILE_REGISTRY
from ..storages.base import BaseStorage
from ..storages.filesystem import FileSystemStorage

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2022-2023 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("RandomFileFromDirProvider",)


class RandomFileFromDirProvider(BaseProvider, FileMixin):
    """Random file from given directory provider.

    Usage example:

    .. code-block:: python

        from faker import Faker
        from faker_file.providers.random_file_from_dir import (
            RandomFileFromDirProvider,
        )

        FAKER = Faker()
        FAKER.add_provider(RandomFileFromDirProvider)

        file = FAKER.random_file_from_dir(
            source_dir_path="/tmp/tmp/",
        )

    Usage example with options:

    .. code-block:: python

        file = FAKER.random_file_from_dir(
            source_dir_path="/tmp/tmp/",
            prefix="zzz",
        )
    """

    extension: str = ""

    @overload
    def random_file_from_dir(
        self: "RandomFileFromDirProvider",
        source_dir_path: str,
        storage: Optional[BaseStorage] = None,
        basename: Optional[str] = None,
        prefix: Optional[str] = None,
        raw: bool = True,
        **kwargs,
    ) -> BytesValue: ...

    @overload
    def random_file_from_dir(
        self: "RandomFileFromDirProvider",
        source_dir_path: str,
        storage: Optional[BaseStorage] = None,
        basename: Optional[str] = None,
        prefix: Optional[str] = None,
        **kwargs,
    ) -> StringValue: ...

    def random_file_from_dir(
        self: "RandomFileFromDirProvider",
        source_dir_path: str,
        storage: Optional[BaseStorage] = None,
        basename: Optional[str] = None,
        prefix: Optional[str] = None,
        raw: bool = False,
        **kwargs,
    ) -> Union[BytesValue, StringValue]:
        """Pick a random file from given directory.

        :param source_dir_path: Source files directory.
        :param storage: Storage. Defaults to `FileSystemStorage`.
        :param basename: File basename (without extension).
        :param prefix: File name prefix.
        :param raw: If set to True, return `BytesValue` (binary content of
            the file). Otherwise, return `StringValue` (path to the saved
            file).
        :return: Relative path (from root directory) of the generated file
            or raw content of the file.
        """
        # Generic
        if storage is None:
            storage = FileSystemStorage()

        # Specific
        source_file_choices = [
            os.path.join(source_dir_path, _f)
            for _f in os.listdir(source_dir_path)
            if os.path.isfile(os.path.join(source_dir_path, _f))
        ]
        source_file_path = choice(source_file_choices)
        source_file = Path(source_file_path)

        # Generic
        filename = storage.generate_filename(
            extension=source_file.suffix[1:],
            prefix=prefix,
            basename=basename,
        )
        data = {"filename": filename, "storage": storage}

        # Specific
        with open(source_file_path, "rb") as _file:
            if raw:
                raw_content = BytesValue(_file.read())
                raw_content.data = data
                return raw_content

            storage.write_bytes(filename, _file.read())

        # Generic
        file_name = StringValue(storage.relpath(filename))
        file_name.data = data
        FILE_REGISTRY.add(file_name)
        return file_name



from faker import Faker
from faker_file.providers.zip_file import ZipFileProvider

FAKER = Faker()
FAKER.add_provider(ZipFileProvider)

zip_file = FAKER.zip_file(
    options={"create_inner_file_args": {"content": "Lorem ipsum"}}
)



from faker import Faker
from faker_file.providers.png_file import GraphicPngFileProvider

FAKER = Faker()
FAKER.add_provider(GraphicPngFileProvider)

png_file = FAKER.graphic_png_file(size=(800, 800))
